@@ -34,19 +34,36 @@ class StrictPricepoints:
 
     @classmethod
     def new(cls, pricepoints: Sequence[Pricepoint]):
-        sorted_pricepoints = cls._sort_pricepoints(pricepoints)
+        unique_pricepoints = cls._make_pricepoints_unique(pricepoints)
+        sorted_pricepoints = cls._sort_pricepoints(unique_pricepoints)
         cls._validate_pricepoints(sorted_pricepoints)
         return cls(sorted_pricepoints)
 
     @classmethod
     def _validate_pricepoints(cls, pricepoints: Sequence[Pricepoint]):
-        # first validate oracle pubkeys (definitely die if mixed oracles)
+        # The point of separating validation and post_init is to allow direct
+        # setting of values as well as using new() constructor.
+
+        # first validate oracle pubkeys (raise an exception in any case if mixed oracles)
         if pricepoints:
             oracle_pubkey = pricepoints[0].oracle_pubkey
             for p in pricepoints:
                 validators.equal(p.oracle_pubkey, oracle_pubkey)
 
-        # enforce uniqueness by timestamp
+        # ensure uniqueness. not efficient but performance is not yet an issue
+        unique_pricepoints = cls._make_pricepoints_unique(pricepoints)
+        validators.equal(pricepoints, unique_pricepoints)
+
+        # ensure sorting. not efficient but performance is not yet an issue
+        sorted_pricepoints = cls._sort_pricepoints(unique_pricepoints)
+        validators.equal(unique_pricepoints, sorted_pricepoints)
+
+    @staticmethod
+    def _sort_pricepoints(pricepoints: Sequence[Pricepoint]) -> tuple[Pricepoint]:
+        return tuple(sorted(pricepoints, key=lambda p: p.timestamp))
+
+    @staticmethod
+    def _make_pricepoints_unique(pricepoints: Sequence[Pricepoint]) -> tuple[Pricepoint]:
         lookup = {}
         for p in pricepoints:
             existing = lookup.get(p.timestamp)
@@ -54,17 +71,11 @@ class StrictPricepoints:
                 lookup[p.timestamp] = p
             else:
                 if p == existing:
-                    raise ValueError(f'duplicate pricepoint in sequence: {p}')
+                    # it's an equal duplicate so just ignore it
+                    pass
                 else:
                     raise ValueError(f'same timestamp, same oracle, different value: {p} vs {existing}')
-
-        # enforce sorting. not efficient but performance is not yet an issue
-        sorted_pricepoints = cls._sort_pricepoints(pricepoints)
-        validators.equal(tuple(pricepoints), sorted_pricepoints)
-
-    @staticmethod
-    def _sort_pricepoints(pricepoints: Sequence[Pricepoint]) -> tuple[Pricepoint]:
-        return tuple(sorted(pricepoints, key=lambda p: p.timestamp))
+        return tuple(lookup.values())
 
     def __str__(self):
         return f'StrictPricepoints: {len(self.all)} prices for {self.all[0].oracleUnit_name}'
@@ -198,6 +209,64 @@ class ScriptPriceInOracleUnitsPerBch(int):
         validators.greater_equal(self, 1)
 
 
+# GP Oracles
+class BtcEM6(OracleUnit):
+    name_of_oracleUnits = 'BTC(e-6)'
+    name_of_standardUnits = 'BTC'
+    oracleUnits_per_standardUnit = 1000000
+    public_key = PublicKey('0245a107de5c6aabc9e7b976f26625b01474f90d1a7d11c180bec990b6938e731e')
+
+
+class CnyEM2(OracleUnit):
+    name_of_oracleUnits = 'CNY(e-2)'
+    name_of_standardUnits = 'CNY'
+    oracleUnits_per_standardUnit = 100
+    public_key = PublicKey('030654b9598186fe4bc9e1b0490c6b85b13991cdb9a7afa34af1bbeee22a35487a')
+
+
+class DogeEM1(OracleUnit):
+    name_of_oracleUnits = 'DOGE(e-1)'
+    name_of_standardUnits = 'DOGE'
+    oracleUnits_per_standardUnit = 10
+    public_key = PublicKey('02461db2aead56ea67543c8196a248152185cb824802a2e7ff12bfc2a493898b8f')
+
+
+class EthEM5(OracleUnit):
+    name_of_oracleUnits = 'ETH(e-5)'
+    name_of_standardUnits = 'ETH'
+    oracleUnits_per_standardUnit = 100000
+    public_key = PublicKey('038ab22e37cf020f6bbef40111ddc51083a936f0821de56ac01f799cf15b87904d')
+
+
+class InrEM0(OracleUnit):
+    name_of_oracleUnits = 'INR'
+    name_of_standardUnits = 'INR'
+    oracleUnits_per_standardUnit = 1
+    public_key = PublicKey('02e82ad82eb88fcdfd02fd5e2e0a67bc6ef4139bbcb63ce0b107a7604deb9f7ce1')
+
+
+class UsdEM2(OracleUnit):
+    name_of_oracleUnits = 'USD(e-2)'
+    name_of_standardUnits = 'USD'
+    oracleUnits_per_standardUnit = 100
+    public_key = PublicKey('02d09db08af1ff4e8453919cc866a4be427d7bfe18f2c05e5444c196fcf6fd2818')
+
+
+class XagEM3(OracleUnit):
+    name_of_oracleUnits = 'XAG(e-3)'
+    name_of_standardUnits = 'XAG'
+    oracleUnits_per_standardUnit = 1000
+    public_key = PublicKey('02712c349ebb7555b17bdbbe9f7aad5a337fa4179d0680eec3f6c8d77bac9cfa79')
+
+
+class XauEM5(OracleUnit):
+    name_of_oracleUnits = 'XAU(e-5)'
+    name_of_standardUnits = 'XAU'
+    oracleUnits_per_standardUnit = 100000
+    public_key = PublicKey('021f8338ccd45a7790025de198a266f252ac43c95bf81d2469feff110beeac89dd')
+
+
+# GP Beta Oracles
 class BtcEM6Beta(OracleUnit):
     name_of_oracleUnits = 'BTC(e-6)'
     name_of_standardUnits = 'BTC'
@@ -240,13 +309,6 @@ class UsdEM2Beta(OracleUnit):
     public_key = PublicKey('02d3c1de9d4bc77d6c3608cbe44d10138c7488e592dc2b1e10a6cf0e92c2ecb047')
 
 
-class UsdEM2Testing(OracleUnit):
-    name_of_oracleUnits = 'USD(e-2)'
-    name_of_standardUnits = 'USD'
-    oracleUnits_per_standardUnit = 100
-    public_key = PublicKey('03e4a5b4a1d9365492dfcad9a05adcdea259acb51a20eabb84584f7aa3c7009ebe')
-
-
 class XagEM3Beta(OracleUnit):
     name_of_oracleUnits = 'XAG(e-3)'
     name_of_standardUnits = 'XAG'
@@ -261,7 +323,19 @@ class XauEM5Beta(OracleUnit):
     public_key = PublicKey('03e980928f14fc98e1f9d75d15f0b67dc58cdd3f5c641b8f825b146bcc04bd232c')
 
 
-known_unit_classes = [BtcEM6Beta, CnyEM2Beta, DogeEM1Beta, EthEM5Beta, InrEM0Beta, UsdEM2Beta, UsdEM2Testing, XagEM3Beta, XauEM5Beta]
+# Very old GP Testing Oracles
+class UsdEM2Testing(OracleUnit):
+    name_of_oracleUnits = 'USD(e-2)'
+    name_of_standardUnits = 'USD'
+    oracleUnits_per_standardUnit = 100
+    public_key = PublicKey('03e4a5b4a1d9365492dfcad9a05adcdea259acb51a20eabb84584f7aa3c7009ebe')
+
+
+known_unit_classes = [
+    BtcEM6, CnyEM2, DogeEM1, EthEM5, InrEM0, UsdEM2, XagEM3, XauEM5,
+    BtcEM6Beta, CnyEM2Beta, DogeEM1Beta, EthEM5Beta, InrEM0Beta, UsdEM2Beta, XagEM3Beta, XauEM5Beta,
+    UsdEM2Testing,
+]
 oracle_pubkey_to_unit_class: dict[PublicKey, Type[OracleUnit]] = {
     u.public_key: u
     for u in known_unit_classes
